
import re
import time
from time import gmtime, strftime
import datetime
import termstatus
import sys
import math
import socket
import time
import pdb
import select
import struct
import netgpib
filetime = strftime("%d%m%Y-%H%M%S", gmtime()) # Date string for data file ; time in gmt(default)
#*************************************************
#EDIT SETTINGS HERE
#Edit these options carefully
#Undefined items may produce undefined or wrong
#results
#*************************************************

ipAddress       = "192.168.1.2" # GPIB IP
gpibAddress     = int(16)       # GPIB port
title           = "HP8560"      # Extra file memo
filename        = filetime      # Add time to filename
memo            = "Noise after shorting LC"     # Extra info if needed

#*************************************************
#Edit these setting for your application
#*************************************************

centerFreq = ""    # MHZ KHZ or HZ fom 30HZ-2.9Ghz
freqSpan   = ""    #

startFreq  = "20MHz"      #  Dependent on Freq span 631KHZ
stopFreq   = "70MHz"     # keep blank if using freqspan
markerFreq = "45MHZ"       # MUST be within range of freq 2.19MHZ
avgeSpec   = "10"           # Number of avg to do


#*************************************************
# Create a netGPIB class object
#*************************************************
print('Connecting to '+str(ipAddress)+' ...'),
gpibObj=netgpib.netGPIB(ipAddress, gpibAddress, '\004',0)
print('done.')

#*************************************************
#Get a board serial number
#*************************************************
ser  = ""
serialString = "0"
while ser != 'y':
    serialString = raw_input ("enter a serial \n")
    ser          = raw_input ('SERIAL NUMBER -  ' + serialString + '\n' + 'enter [n] or [y] to continue \n')
print "Serial Saved"

# File names
# added filetime to file name
dataFileName=serialString+' '+filetime+'.txt'
print('Board Serial = '+serialString)
print('File time    = '+filetime)
print('Measurement data will be written into '+dataFileName)
print('Setting up parameters for the measurement...')

#*************************************************
#Send data to GPIB
#*************************************************
gpibObj.command("IP;SNGLS;TS;") #Preset the device and set for Single sweep (TS = trace update) 
time.sleep(0.2)
if freqSpan != "":  # use either span and center or start and stop points
      gpibObj.command("CF "+centerFreq+";"+"SP "+freqSpan+";TS;") # Freq span/center
      time.sleep(0.2)
else:
      gpibObj.command("FA "+startFreq+";"+"FB "+stopFreq+";TS;") # start/stop Freq
      time.sleep(0.2)
gpibObj.command("VAVG "+avgeSpec+";"+"TS;") #Preset the device and set for Single sweep 
time.sleep(0.2)
gpibObj.command("MKN "+markerFreq+";") #Preset the device and set for Single sweep 
time.sleep(0.2)
gpibObj.command("TDF P;VIEW TRA;TS;")  # Trace Format "P" view trace
time.sleep(4)

print"Downloading data from display"
gpibObj.query("TDF P;MKA?;")
markerRead = gpibObj.getData()
time.sleep(0.4)

gpibObj.command("VAVG?") #Preset the device and set for Single sweep 
avgeRead = gpibObj.getData()
time.sleep(0.4)

gpibObj.query("TDF P;MKF?;")
markerfreqRead = gpibObj.getData()
time.sleep(0.4)

gpibObj.query("TDF P;CF?;")
centerFreq = gpibObj.getData()
time.sleep(0.4)

gpibObj.query("TDF P;RB?;") 
bandwRead = gpibObj.getData()
time.sleep(0.4)

gpibObj.query("TDF P;Rl?;")  
refLev = gpibObj.getData()
time.sleep(0.4)

gpibObj.query("TDF P;AUNITS?;") 
ampunits = gpibObj.getData()
time.sleep(0.4)
#*************************************************
#Save data to file
#*************************************************
if title == "":
    title = filename
dataFile = open(dataFileName,'w')    
dataFile.write('# HP8560 Spectrum Analysis\n')
dataFile.write('Title: '+title+'\n')
dataFile.write('Memo: '+memo+'\n')
dataFile.write('#---------- User Measurement Setup ------------\n')
dataFile.write('center frequency: '+centerFreq+'\n')
dataFile.write('Frequency Span  : '+freqSpan+'\n')
dataFile.write('Start frequency : '+startFreq+'\n')
dataFile.write('Stop frequency  : '+stopFreq+'\n')
dataFile.write('Marker frequency: '+markerFreq+'\n')
dataFile.write('Averaging       : '+avgeSpec+'\n')
dataFile.write('\n')
dataFile.write('#---------- System read Setup ------------\n')
dataFile.write('Center frequency: '+centerFreq+'\n')
dataFile.write('Refernce Level  : '+refLev+'\n')
dataFile.write('Bandwidth       : '+bandwRead+'\n')
dataFile.write('Amplitude units : '+ampunits+'\n')
dataFile.write('Marker Amplitude: '+markerRead+'\n')
dataFile.write('Marker frequency: '+markerfreqRead+'\n')
dataFile.write('Averaging       : '+avgeRead+'\n')
dataFile.write('\n')
dataFile.write('Amplitude--------------------------\n')
#*************************************************
# Trace Data to be downloaded
#*************************************************
data=[]
gpibObj.query("TDF P;TRA?;")
time.sleep(3)
data = str(gpibObj.getData())
time.sleep(3)
print "download done"
print('Writing measurement data to file...')
datastring =""
printlist  = []
for i in range(len(data)):
      if data[i] != ",":
         datastring+=data[i]
      else:
         printlist.append(datastring)
         dataFile.write(datastring +',' + '\n')
         datastring = ""
#*************************************************
#Save data to file close file/gpib
#*************************************************
dataFile.close()
gpibObj.close()

