import sys
import math
import socket
import time
import pdb
import select
import struct
import termstatus

class netGPIB:
    def __init__(self,ip,gpibAddr,eot='\004',debug=0, auto=False):

        #End of Transmission character
        self.eot=eot
        # EOT character number in the ASCII table
        self.eotNum=struct.unpack('B',eot)[0]

        #Debug flag
        self.debug = debug

        #Auto mode
        self.auto = auto

        self.ip = ip
        self.gpibAddr=gpibAddr

        #Connect to the GPIB-Ethernet converter
        netAddr=(ip, 1234)
        self.netSock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.netSock.connect(netAddr)

        #Initialize the GPIB-Ethernet converter
        self.netSock.setblocking(0)
        self.netSock.send("++addr "+str(self.gpibAddr)+"\n") # set primary add
        time.sleep(0.1)
        self.netSock.send("++eos 3\n") #Do not append anything to instrument commands
        time.sleep(0.1)
        self.netSock.send("++mode 1\n") #Switch to CONTROLLER mode
        time.sleep(0.1)
       # self.netSock.send("++auto 1\n") #Turn on read-after-write and address instrument to listen
        self.netSock.send("++auto 0\n") #Turn off read-after-write and address instrument to listen
        time.sleep(0.1)
        self.netSock.send("++ifc\n") #controller the Controller-In-Charge on the GPIB bus.
        time.sleep(0.1)
        self.netSock.send("++read_tmo_ms 4000\n")
        time.sleep(0.1)
        self.netSock.send("++eot_char "+str(self.eotNum)+"\n") #append '*' as EOT item
        self.netSock.send("++eot_enable 0\n") #("++eot_enable 1\n")
        self.netSock.send("++addr "+str(self.gpibAddr)+"\n")
        
    def refresh(self):
        self.netSock.send("++addr "+str(self.gpibAddr)+"\n")
        time.sleep(0.1)
        self.netSock.send("++eos 3\n")
        time.sleep(0.1)
        self.netSock.send("++mode 1\n")
        time.sleep(0.1)
        if self.auto:
            self.netSock.send("++auto 1\n")
        else:
            self.netSock.send("++auto 0\n")
        time.sleep(0.1)
        self.netSock.send("++ifc\n")
        time.sleep(0.1)
        self.netSock.send("++read_tmo_ms 3000\n")
        time.sleep(0.1)
        self.netSock.send("++eot_char"+str(self.eotNum)+"\n")
        self.netSock.send("++eot_enable 1\n")
        self.netSock.send("++addr "+str(self.gpibAddr)+"\n")


    def getData(self):
        print 'receiving data...'
        self.netSock.send("++read eoi\n")
        time.sleep(5) 
        data =""
        data1 = self.netSock.recv(66999)  # BIG buffer!
        data = data + data1
        return data
            
    def query(self,string,buf=600,sleep=0):
        self.netSock.send(string+"\n")
        time.sleep(sleep)
        #self.netSock.send("++read eoi\n") #Change to listening mode
        
    
    def command(self,string,sleep=0):
        """Send a command to the device."""
        self.netSock.send(string+"\n")
        time.sleep(sleep)

    def spoll(self):
        """Perform a serial polling and return the result."""
        self.netSock.send("++spoll\n")
        while 1:  # Read some data
            readSock, writeSock, errSock = select.select([self.netSock],[],[],3)
            if len(readSock) == 1:
                data = readSock[0].recv(100)
                break

        return data[:-2]
    
    def close(self):
        self.netSock.close()
        
    def setDebugMode(self, debugFlag):
        if debugFlag:
            self.debug=1
        else:
            self.debug=0

  
