#!/usr/bin/env python
#
# FFT/Noise measurement script for suspensions electronics testing (SR785).  Modified from
# code by Yoichi Aso.
# cmreed 20100329
# Edited by Carl Adams for LLO testing on windows machine 20100406
# reedieted by Edward R                
import re
import sys
import math
import optparse
import time
from time import gmtime, strftime
import datetime
import netgpib
import SR785
import termstatus


"""
This version is edited for use on a Win XP OS.
This program remotely executes a spectrum measurement using SR785.
Various measurement conditions can be set using options.
The measurement result will be saved in FILENAME.txt
"""

filetime = strftime("%d%m%Y-%H%M%S", gmtime()) # Date string for data file ; time in gmt(default)

#*************************************************
# EDIT SETTINGS HERE
#Edit these options carefully
#Undefined items may produce undefined or wrong
#results
#*************************************************

ipAddress       = "192.168.1.2"   # Ip of GPIB Device
gpibAddress     = int(10)         # port of GPIB device (10) is default 
filename        = filetime        # Add time to filename
memo            = ""              # Use this option to note miscellaneous things.
title           = "TSFRSR785"     # Title of the measurement.

#***********************************
dualchannel     = "single"        # "single"/"dual" chan mode
numOfPoints     = int(400)        # FFT lines 800 , 400, 200, 100
bandWidth       = "110Hz"         # BandWidth - You can use mHz,Hz or kHz units."
numAvg          = "10"            # Number of averages 0 to n
avgMode         = "RMS"           # Averaging mode: "None, Vector, RMS or PeakHold"
inputCoupling1  = "DC"            # CH1 input coupling. "DC" or "AC"
inputDiff1      = "A"             # CH1 input Differential "A-B", or single ended "A"
inputCoupling2  = "AC"            # CH2 input coupling. "DC" or "AC"
inputGND1       = "Float"         # CH1 input grounding. "Float" or "Ground"
inputGND2       = "Float"         # CH2 input grounding. "Float" or "Ground"
windowFunc      = "BMH"           # Window function: "Uniform, Flattop, Hanning, BMH, Kaiser,
                                  #  -Force/Exponential, User, [-T/2,T/2],[0,T/2] or [-T/4,T/4]
spanFreq        = "3.2kHz"        # Frequency span has only select values avalible
                                  #  - consult the manual or SR785 options menu
startFreq       = "22.8kHz"       # Start frequency 102.4Khz Max
dataMode        = "dbVrms/Hz"     # "dbVrms/Hz"  or "Vrms/Hz" 
baseFreq        = "100.0kHz"      # Base Freq "102.4kHz" or "100.0kHz" 


#*************************************************
# Start
#*************************************************

#Convert number of points into available resolution
if numOfPoints <= 100:
    fRes=0 # Resolution is 100 points
elif numOfPoints <= 200:
    fRes=1 # Resolution is 200 points
elif numOfPoints <= 400:
    fRes=2 # Resolution is 400 points
else:
    fRes=3 # Resolution is 800 points
# Create a netGPIB class object
print('Connecting to '+str(ipAddress)+' ...')
gpibObj=netgpib.netGPIB(ipAddress, gpibAddress, '\004',0)
print('done.')


#*************************************************
#Get a board serial number
#*************************************************
ser  = ""
serialString = "0"
while ser != 'y':
    serialString = raw_input ("enter a serial \n")
    ser          = raw_input ('SERIAL NUMBER -  ' + serialString + '\n' + 'enter [y] or [n] to continue \n')
print "Serial Saved"

# File names
# added filetime to file name
dataFileName=serialString+' '+filetime+' FFT '+'.txt'
print('Board Serial = '+serialString)
print('File time    = '+filetime)
print('Measurement data will be written into '+dataFileName)
print('Setting up parameters for the measurement...') 

#*************************************************
#Prepare for the FFT measurement
#*************************************************

#Set output to GPIB
gpibObj.command("OUTX0")

# Print IDN
print "Instrument ID: ",
idnString=gpibObj.query('*IDN?')
print idnString[:-1]

#Set measurement parameters
if dualchannel == "dual":
    gpibObj.command('DFMT1') # Dual display
    numDisp=2
else:
    gpibObj.command('DFMT0') # single display
    numDisp=1
#Input setup
if inputCoupling1 == "AC":
    icp1="1"
else:
    icp1="0"
gpibObj.command('I1CP'+icp1) #CH1 Input Coupling


if inputCoupling2 == "AC":
    icp2="1"
else:
    icp2="0"
gpibObj.command('I2CP'+icp2) #CH2 Input Coupling


if inputGND1 == "Float":
    igd1="0"
else:
    igd1="1"
gpibObj.command('I1GD'+igd1) #CH1 Input GND


if inputGND2 == "Float":
    igd2="0"
else:
    igd2="1"
gpibObj.command('I2GD'+igd2) #CH2 Input GND


#*************************************************
#Adding option for differental input
#*************************************************

if inputDiff1 == "A":
    idf1="0"
else:
    idf1="1"
gpibObj.command('I1MD'+idf1) #CH1 Input A-B = 1; A = 0
gpibObj.command('A1RG0') #AutoRange Off
gpibObj.command('A2RG0') #AutoRange Off
gpibObj.command('I1AR0') #AutoRange Up Only
gpibObj.command('I2AR0') #AutoRange Up Only
gpibObj.command('A1RG1') #AutoRange On
gpibObj.command('A2RG1') #AutoRange On
gpibObj.command('I1AF1') #Anti-Aliasing filter On
gpibObj.command('I2AF1') #Anti-Aliasing filter On
#Common setup for both displays
gpibObj.command('MGRP2,0') # Measurement Group = FFT
gpibObj.command('ISRC1')   # Input = Analog

if baseFreq == "102.4kHz":
   gpibObj.command('FBAS2,1')  # Base Frequency = 102.4kHz
else:
    gpibObj.command('FBAS2,0') # Base Frequency = 100.0kHz

#*************************************************   
#Measurement Types to Set up each display
#*************************************************
   
if  dataMode == "dbVrms/Hz": 
    for disp in range(numDisp):
     gpibObj.command('UNDB'+str(disp)+','+str(1))   # dB ON
     gpibObj.command('UNPK'+str(disp)+','+str(0))   # Vrms OFF
else:
    for disp in range(numDisp):
     gpibObj.command('UNDB'+str(disp)+','+str(0))   # dB OFF
     gpibObj.command('UNPK'+str(disp)+','+str(2))   # Vrms ON

for disp in range(numDisp):
    gpibObj.command('ACTD'+str(disp)) # Change active display 
    gpibObj.command('MEAS'+str(disp)+','+str(disp)) # 0:FFT1, 1:FFT2    
    gpibObj.command('VIEW'+str(disp)+',0') #Log Magnitude
    gpibObj.command('PSDU'+str(disp)+',1') # PSD ON
    gpibObj.command('DISP'+str(disp)+',1') # Live display on
    
#*************************************************   
#Common setup
#*************************************************
gpibObj.command('FLIN2,'+str(fRes))     # Frequency resolution
gpibObj.command('FAVG2,1')              # Averaging On

avgModDict = {"None":0, "Vector":1, "RMS":2, "PeakHold":3}
if avgMode in avgModDict:
    avgModID=avgModDict[avgMode]
else:
    avgModID=2
gpibObj.command('FAVM2,'+str(avgModID)) # Averaging mode
gpibObj.command('FAVT2,0')            # Averaging Type = Linear
gpibObj.command('FREJ2,1')            # Overload Reject On
gpibObj.command('FAVN2,'+str(numAvg)) # Number of Averaging
winFuncDict = {"Uniform":0, "Flattop":1, "Hanning":2, "BMH":3, "Kaiser":4, "Force/Exponential":5, "User":6, "[-T/2,T/2]":7,"[0,T/2]":8,"[-T/4,T/4]":9}

if windowFunc in winFuncDict:
    winFuncID=winFuncDict[windowFunc]
else:
    winFuncID=2
gpibObj.command('FWIN2,'+str(winFuncID))    # Window function
gpibObj.command('FSTR2,'+startFreq)         # Start frequency
gpibObj.command('FSPN2,'+spanFreq)          # Frequency span


# gpibObj.command('MEAS'+str(disp)+',3') # Measure 'Power Spectrum 2'
time.sleep(1)

#*************************************************
#Start measurement
#*************************************************
print 'Measurement started'
sys.stdout.flush()
gpibObj.command('STRT') #Start measurement
#Wait for the measurement to end
measuring = True
avg=0
print 'Averaging completed:'
avgStatus=termstatus.statusTxt("0")
while measuring:
    #Get status 
    measuring = not int(gpibObj.query('DSPS?1'))
    avg=int(gpibObj.query("NAVG?0"))
    avgStatus.update(str(avg))
        
    #if  numAvg <= avg:
    #    measuring = False
    time.sleep(0.3)

a=int(gpibObj.query("NAVG?0"))
avgStatus.end(str(a))
print('done')

gpibObj.command('ASCL0') #Auto scale
gpibObj.command('ASCL1') #Auto scale


#Download Data
time.sleep(0.5)

data=[]
for disp in range(numDisp):
    print('Downloading data from display #'+str(disp))
    (f,d)=SR785.downloadData(gpibObj, disp)        
    data.append(d)
    
freq=f

# Data index for 10Hz
#FI=freq.index('+1.0000000e+001')

# Print a summary table to fill in the test document
#print '----- Noise Test Summary at 10Hz -----'
#print 'Freq(Hz)\tMag(V/rtHz)'
#print str(float(freq[FI]))+'\t\t'+str(float(data[0][FI]))
#print '----------------------------------------'


# Open file
dataFile = open(dataFileName,'w')
print('Writing measurement data to file...')

#Deal with an empty title
if title == "":
    title = filename

#Write parameters to file
dataFile.write(filetime)
dataFile.write('# SR785 SUS Noise Measurement\n')
dataFile.write('Title: '+title+'\n')
dataFile.write('Memo: '+memo+'\n')
dataFile.write('Instrument ID: '+idnString+'\n')
dataFile.write('Board Serial ID: '+serialString+'\n')
dataFile.write('#---------- Measurement Setup ----------\n')
fSpan=gpibObj.query("FSPN?0")
fSpan=fSpan[:-1]
dataFile.write('Frequency Span: '+fSpan+'\n')

fRes=int(gpibObj.query("FLIN?0"))
fRes={0:100, 1:200, 2:400, 3:800}[fRes]
dataFile.write('Frequency Resolution: '+str(fRes)+'\n')

nAvg=int(gpibObj.query("NAVG?0"))
dataFile.write('Number of Averages: '+str(nAvg)+'\n')

avgMode={0:"None", 1:"Vector", 2:"RMS", 3:"PeakHold"}[int(gpibObj.query("FAVM?0"))]
dataFile.write('Averaging Mode: '+avgMode+'\n')

winFunc={0:"Uniform", 1:"Flattop", 2:"Hanning", 3:"BMH", 4:"Kaiser", 5:"Force/Exponential", 6:"User", 7:"[-T/2,T/2]",8:"[0,T/2]",9:"[-T/4,T/4]"}[int(gpibObj.query("FWIN?0"))]
dataFile.write('Window function: '+winFunc+'\n')

dataFile.write('\n')

SR785.getparam(gpibObj, filename, dataFile)

dataFile.write('#---------- Instrument Data -----------\n')
dataFile.write('# Column format = [Freq  Mag(V/rtHz)]\n')

#Write data to file
for i in range(len(freq)):
    dataFile.write(freq[i])
    dataFile.write(' '+data[0][i])
    dataFile.write('\n')
dataFile.close()
print "done."

gpibObj.close()

