#!/usr/bin/env python
#
# TSFR measurement script for suspensions electronics testing (SR785).  Modified from
# code by Yoichi Aso.
# cmreed 20100329
# Edited by Carl Adams for LLO testing on windows machine 20100406
# Reedited by Edward R    
import re
import sys
import math
import optparse
import time
from time import gmtime, strftime
import datetime
import pdb
import netgpib
import SR785
import termstatus

'''This version is edited for use on a Win XP OS.
This program executes a transfer function measurement using SR785.
Various measurement conditions can be set using options.
The measurement results will be saved in FILENAME.txt.
'''
filetime = strftime("%d%m%Y-%H%M%S", gmtime()) # Date string for data file ; time in gmt(default)

#*************************************************
# EDIT SETTINGS HERE
#Edit these options carefully
#Undefined items may produce undefined or wrong
#results
#*************************************************
ipAddress       = "192.168.1.2" # Ip of GPIB Device
gpibAddress     = int(10)       # port of GPIB device (10) is deafult 
filename        = filetime      # Add time to filename
memo            = ""            # Use this option to note miscellaneous things.
title           = "TSFRSR785"   # Title of the measurement.
#*****************************************
startFreq       = "10.9kHz"     #Start frequency. Units can be mHz, Hz or kHz.
stopFreq        = "33.3kHz"     #Stop frequency. Units can be mHz, Hz or kHz.
#####  changing freq range requires editing the index, summary table, and dataFile.write(freq[...]) below (near bottom)! ######
numOfPoints     = int(101)      #Number of frequency points"
sweepType       = "Log"         #Sweep type: Log or Linear (default: Log)
excAmp          = "2500mV"      #Excitation amplitude
settleCycles    = int(1)        #Settle cycles
intCycles       = int(10)       #Integration cycles
inputCoupling1  = "AC"          #CH1 input coupling. DC or AC (default:AC)
inputCoupling2  = "AC"          #CH2 input coupling. DC or AC (default:AC)
inputGND1       = "Float"       #CH1 input grounding. Float or Ground (default:Float)
inputGND2       = "Float"       #CH2 input grounding. Float or Ground (default:Float)
arMode          = "UpOnly"      #Auto range mode: UpOnly or Tracking (default: UpOnly)
correction      = float(26.0)   #Correction to apply to data in dB (formula: DATA - CORRECTION, default: 26.0)
inputDiff1      = "A-B"         #CH1 input Differential A-B, or single ended A"


#*************************************************
#GPIB setup and connection
#*************************************************
print('Connecting to '+str(ipAddress)+' ...'),
gpibObj=netgpib.netGPIB(ipAddress, gpibAddress, '\004',0)
print('done.')


#*************************************************
#Get a board serial number
#*************************************************
ser  = ""
serialString = "0"
while ser != 'y':
    serialString = raw_input ("enter a serial \n")
    ser          = raw_input ('SERIAL NUMBER -  ' + serialString + '\n' + 'enter [n] or [y] to continue \n')
print "Serial Saved"

# added filetime to file name
dataFileName=serialString+' '+filetime+'.txt'
print('Board Serial = '+serialString)
print('File time    = '+filetime)
print('Measurement data will be written into '+dataFileName)
print('Setting up parameters for the measurement...')


#*************************************************
#Prepare for the TF measurement
#*************************************************

#Set output to GPIB
gpibObj.command("OUTX0")

# Print IDN
print "Instrument ID: ",
idnString=gpibObj.query("*IDN?")
print idnString[:-1]
time.sleep(0.1)

#Input setup
if inputCoupling1 == "AC":
    icp1="1"
else:
    icp1="0"
gpibObj.command('I1CP'+icp1) #CH1 Input Coupling

if inputCoupling2 == "AC":
    icp2="1"
else:
    icp2="0"
gpibObj.command('I2CP'+icp2) #CH2 Input Coupling

if inputGND1 == "Float":
    igd1="0"
else:
    igd1="1"
gpibObj.command('I1GD'+igd1) #CH1 Input GND


if inputGND2 == "Float":
    igd2="0"
else:
    igd2="1"
gpibObj.command('I2GD'+igd2) #CH2 Input GND
gpibObj.command('A1RG0') #AutoRange Off
gpibObj.command('A2RG0') #AutoRange Off
if arMode == "Tracking":
    arModeID='1'
else:
    arModeID='0'
gpibObj.command('I1AR'+arModeID) #Auto Range Mode 
gpibObj.command('I2AR'+arModeID) #Auto Range Mode 
gpibObj.command('A1RG1') #AutoRange On
gpibObj.command('A2RG1') #AutoRange On
gpibObj.command('I1AF1') #Anti-Aliasing filter On
gpibObj.command('I2AF1') #Anti-Aliasing filter On

###########____Adding option for differental input____############
if inputDiff1 == "A":
    idf1="0"
else:
    idf1="1"
gpibObj.command('I1MD'+idf1) #CH1 Input A-B = 1; A = 0

#Set measurement parameters
gpibObj.command('DFMT1') # Dual display
gpibObj.command('ACTD0') # Active display 0
gpibObj.command('MGRP2,3') # Measurement Group = Swept Sine
gpibObj.command('MEAS2,47') # Frequency Resp
gpibObj.command('VIEW0,0') # Disp 0 = LogMag
gpibObj.command('VIEW1,5') # Dsip 1 = Phase
gpibObj.command('UNDB0,1') # dB ON
gpibObj.command('UNPK0,0') # PK Unit Off
gpibObj.command('UNDB1,0') # dB OFF
gpibObj.command('UNPK1,0') # PK Unit Off
gpibObj.command('UNPH1,0') # Phase Unit deg.
gpibObj.command('DISP0,1') # Live display on
gpibObj.command('DISP1,1') # Live display on
gpibObj.command('SSCY2,'+str(settleCycles)) # Settle cycles
gpibObj.command('SICY2,'+str(intCycles)) # Integration cycles
gpibObj.command('SSTR2,'+startFreq) #Start frequency
gpibObj.command('SSTP2,'+stopFreq) #Stop frequency
gpibObj.command('SNPS2,'+str(numOfPoints)) #Number of points
gpibObj.command('SRPT2,0') #Single shot mode
if sweepType == 'Linear':
    sweepTypeID='0'
else:
    sweepTypeID='1'
gpibObj.command('SSTY2,'+sweepTypeID) # Sweep Type
gpibObj.command('SSAM'+excAmp) #Source Amplitude


#Start measurement
print 'Transfer function measurement started:'
sys.stdout.flush()
numPoints=int(gpibObj.query('SNPS?0')) #Number of points
gpibObj.command('STRT') #Source Amplitude


#Wait for the measurement to end
measuring = True
percentage=0
accomplished=0
progressInfo=termstatus.statusTxt('0%')
while measuring:
    #Get status 
    measuring = not int(gpibObj.query('DSPS?4'))
    a=int(gpibObj.query('SSFR?'))
    percentage=int(math.floor(100*a/numPoints))
    progressInfo.update(str(percentage)+'%')
    accomplished=percentage
    time.sleep(0.3)
progressInfo.end('100%')

#Download Data
time.sleep(2)

data=[]
for disp in range(2):
    print('Downloading data from display #'+str(disp))
    (f,d)=SR785.downloadData(gpibObj, disp)        
    data.append(d)
    
freq=f

# Convert dB data to float
floatdata = [float(item) for item in data[0]]
# Correct dB data using options
caldata = map((lambda x: x-correction), floatdata)
# Cast float data back to string
strcaldata = map((lambda x: "%e" % x), caldata)

# Data indexes for 0.1Hz to 100kHz
#FI1=freq.index('+1.0000000e-001')
#FI2=freq.index('+1.0000000e+000')
#FI3=freq.index('+1.0000000e+001')
#FI4=freq.index('+1.0000000e+002')
#FI5=freq.index('+1.0000000e+003')
#FI6=freq.index('+1.0000000e+004')
#FI7=freq.index('+1.0000000e+005')
correctionString=str(correction)

# Print a summary table to fill in the test document
print '----- Transfer Function Test Summary -----'
print 'Calibrated Mag(dB) = Mag(dB) - '+correctionString+'(dB)'
print
print 'Freq(Hz)\tMag(dB)\t\tCalibratedMag(dB)\tPhase(deg)'
#print str(float(freq[FI1]))+'\t\t'+str(float(data[0][FI1]))+'\t'+str(caldata[FI1])+'\t\t'+str(float(data[1][FI1]))
#print str(float(freq[FI2]))+'\t\t'+str(float(data[0][FI2]))+'\t'+str(caldata[FI2])+'\t\t'+str(float(data[1][FI2]))
#print str(float(freq[FI3]))+'\t\t'+str(float(data[0][FI3]))+'\t'+str(caldata[FI3])+'\t\t'+str(float(data[1][FI3]))
#print str(float(freq[FI4]))+'\t\t'+str(float(data[0][FI4]))+'\t'+str(caldata[FI4])+'\t\t'+str(float(data[1][FI4]))
#print str(float(freq[FI5]))+'\t\t'+str(float(data[0][FI5]))+'\t'+str(caldata[FI5])+'\t\t'+str(float(data[1][FI5]))
#print str(float(freq[FI6]))+'\t\t'+str(float(data[0][FI6]))+'\t'+str(caldata[FI6])+'\t\t'+str(float(data[1][FI6]))
#print str(float(freq[FI7]))+'\t\t'+str(float(data[0][FI7]))+'\t'+str(caldata[FI7])+'\t\t'+str(float(data[1][FI7]))
print '----------'


#Open file
dataFile = open(dataFileName,'w')

#Write to the data file
print('Writing measurement data to file...')

#Deal with an empty title
if title == "":
     title = filename

#Write parameters to file
dataFile.write('# SR785 SUS Transfer Function Measurement\n')
dataFile.write('Title: '+title+'\n')
dataFile.write('Memo: '+memo+'\n')
dataFile.write('Instrument ID: '+idnString+'\n')
dataFile.write('#---------- Measurement Setup ------------\n')
dataFile.write('Start frequency = '+startFreq+'\n')
dataFile.write('Stop frequency = '+stopFreq+'\n')
dataFile.write('Number of frequency points = '+str(numOfPoints)+'\n')
dataFile.write('Excitation amplitude = '+excAmp+'\n')
dataFile.write('Settling cycles = '+str(settleCycles)+'\n')
dataFile.write('Integration cycles = '+str(intCycles)+'\n')
dataFile.write('\n')

SR785.getparam(gpibObj, filename, dataFile)

dataFile.write('#---------- Transfer Function Test Summary -----------\n')
dataFile.write('Calibrated Mag(dB) = Mag(dB) - '+correctionString+'(dB)\n')
dataFile.write('[Frequency (Hz) Mag(dB) CalibratedMag(db) Phase(deg)]\n')
#dataFile.write(freq[FI1]+' '+data[0][FI1]+' '+strcaldata[FI1]+' '+data[1][FI1]+'\n')
#dataFile.write(freq[FI2]+' '+data[0][FI2]+' '+strcaldata[FI2]+' '+data[1][FI2]+'\n')
#dataFile.write(freq[FI3]+' '+data[0][FI3]+' '+strcaldata[FI3]+' '+data[1][FI3]+'\n')
#dataFile.write(freq[FI4]+' '+data[0][FI4]+' '+strcaldata[FI4]+' '+data[1][FI4]+'\n')
#dataFile.write(freq[FI5]+' '+data[0][FI5]+' '+strcaldata[FI5]+' '+data[1][FI5]+'\n')
#dataFile.write(freq[FI6]+' '+data[0][FI6]+' '+strcaldata[FI6]+' '+data[1][FI6]+'\n')
dataFile.write('\n')

dataFile.write('#---------- Instrument Data -----------\n')
dataFile.write('# Column format = [Freq  Mag(dB) CalibratedMag(dB) Phase(deg)]\n')

#Write data to file
for i in range(len(freq)):
    dataFile.write(freq[i])
    dataFile.write(' '+data[0][i])
    dataFile.write(' '+strcaldata[i])
    dataFile.write(' '+data[1][i])
    dataFile.write('\n')

dataFile.close()
print "done."
gpibObj.close()

